/*
 * some commonly used utilities
 *
 * Choi Jonghwan <saxboy@freechal.com>
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <gnome.h>
#include <signal.h>
#include <ctype.h>

#include "util.h"

int file_Exist(const char *filename)
{
	struct stat stbuf; 
	if ( stat(filename, &stbuf) < 0 )
		return 0; 

	return 1; 
}

void _confirm_no_blanks(char *buf, int len)
{
	int i=0; 

	/* FIXME: scanning from the end of the string is better. 
	 */
	for (i=0; i<len; i++)
	{
		if ( isspace(buf[i]) )
		{
			buf[i]='\0'; 
			return; 
		}
	}
}

int parse_gnome_desktop_file(
		const char *file, 
		char *iconpath, int iconpath_len, 
		char *execpath, int execpath_len)
{
	FILE *fp; 
	char buf[1024]={0,}; 


	if (file==NULL) return -1; 
	if (iconpath==NULL) return -1; 
	if (execpath==NULL) return -1; 

	if ( (fp=fopen(file, "r"))==NULL) return -1; 
	while ( fgets( buf, 1024, fp)!=NULL) 
	{
		/* get lines starting with Exec & Icon 
		 */
		if ( strncmp(buf, "Exec=", 5)==0 )
		{
			strncpy( execpath, &buf[5], execpath_len ); 
			_confirm_no_blanks(execpath, execpath_len); 
		}
		else if ( strncmp(buf, "Icon=", 5)==0 )
		{
			strncpy( iconpath, &buf[5], iconpath_len ); 
			_confirm_no_blanks(iconpath, iconpath_len); 
			strncpy( iconpath, gnome_pixmap_file(iconpath), iconpath_len ); 
		}

	}

	fclose(fp); 

	return 1; 
}

GtkWidget *gtk_button_new_with_imagefile(gchar *imgfile)
{
	GtkWidget *button; 
	GtkWidget *picture; 

	button = gtk_button_new(); 

	if ( file_Exist(imgfile)  )
	{
		picture = get_picture_pixmap(imgfile); 
		gtk_container_add(GTK_CONTAINER(button), picture); 
	}
	else 
	{
		printf("%s: imgfile [%s] does not exist\n", __FUNCTION__, imgfile); 
	}

	return button; 
}

GtkWidget *get_picture_pixmap_with_size( 
				const char* imgfile, int wreq, int hreq)
{
    GtkWidget *pixmap;
	GdkImlibImage *im; 
	GdkPixmap	*p, *m; 
	gint w, h; 

	if ( (im = gdk_imlib_load_image((char *)imgfile))==NULL)
		return NULL; 

	w=wreq; h=hreq; 

	gdk_imlib_render(im, w, h); 
	if ( (p = gdk_imlib_move_image(im))!=NULL)
		gdk_pixmap_ref(p); 
	if ( (m = gdk_imlib_move_mask(im))!=NULL)
		gdk_pixmap_ref(m); 

	pixmap = gtk_pixmap_new(p, m); 
	gtk_widget_show(pixmap); 

	gdk_imlib_destroy_image(im); 

	return pixmap; 
}


GtkWidget *get_picture_pixmap( const char* imgfile)
{
    GtkWidget *pixmap;
	GdkImlibImage *im; 
	GdkPixmap	*p, *m; 
	gint w, h; 

//	gdk_imlib_init(); 
//	gtk_widget_push_visual(gdk_imlib_get_visual()); 
//	gtk_widget_push_colormap(gdk_imlib_get_colormap()); 

	if ( (im = gdk_imlib_load_image((char *)imgfile))==NULL)
		return NULL; 

	w=im->rgb_width; h=im->rgb_height; 

	gdk_imlib_render(im, w, h); 
	if ( (p = gdk_imlib_move_image(im))!=NULL)
		gdk_pixmap_ref(p); 
	if ( (m = gdk_imlib_move_mask(im))!=NULL)
		gdk_pixmap_ref(m); 

	pixmap = gtk_pixmap_new(p, m); 
	gtk_widget_show(pixmap); 

	gdk_imlib_destroy_image(im); 

	return pixmap; 
}

#if 0
GtkWidget *xpm_label_box( gchar     *xpm_filename,
                          gchar     *label_text )
{
    GtkWidget *box;
    GtkWidget *label;
    GtkWidget *image;

    /* Create box for image and label */
    box = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (box), 2);

    /* Now on to the image stuff */
    image = gtk_image_new_from_file (xpm_filename);

    /* Create a label for the button */
    label = gtk_label_new (label_text);

    /* Pack the image and label into the box */
    gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 3);
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 3);

    gtk_widget_show (image);
    gtk_widget_show (label);

    return box;
}

#endif

void show_message(gchar *message) {

   GtkWidget *dialog, *okay_button;
   
   /* Create the widgets */
   
   //dialog = gtk_dialog_new();
   dialog = gtk_window_new(GTK_WINDOW_POPUP);
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER); 
	gtk_window_set_modal(GTK_WINDOW(dialog), TRUE); 
   //label = gtk_label_new (message);
   okay_button = gtk_button_new_with_label(message);
   
   /* Ensure that the dialog box is destroyed when the user clicks ok. */
   
   gtk_signal_connect_object (GTK_OBJECT (okay_button), "clicked",
                              GTK_SIGNAL_FUNC (gtk_widget_destroy), 
							  (gpointer) dialog);
   gtk_container_add (GTK_CONTAINER (GTK_WINDOW(dialog)),
                      okay_button);

   /* Add the label, and show everything we've added to the dialog. */

   gtk_widget_show_all (dialog);
}

gchar *get_selected_chars(GtkWidget *widget)
{
    GtkEditable *editable = NULL;
	gchar *selection;

	if ( !GTK_IS_EDITABLE(widget) )
		return NULL; 

    if ( ! GTK_EDITABLE(widget)->has_selection)
		return NULL; 

    if ( (editable= GTK_EDITABLE(widget)) ==NULL ) 
		return NULL; 

    {
        int start, end;

        start = MIN(editable->selection_start_pos, editable->selection_end_pos);
        end = MAX(editable->selection_start_pos, editable->selection_end_pos);
        selection = gtk_editable_get_chars(editable, start, end);

    }

	return selection;
}



#include <signal.h>
#include <sys/wait.h>
static void handle_sigchld(int signo)
{
    int status; 
    waitpid( -1, &status, WNOHANG );

    printf("status=%d\n", status);
    

    if (status != 0 )
    {
        //show_message( MV_TEXT_FAILED ); 
    }
}


int execute_command_background(const char *command, pid_t *pid_ptr)
{
    /* launch application 
     */
    int pid=0;

    if (command==NULL) return -1;
printf("exec----->[%s]\n", command);

    signal( SIGCHLD, handle_sigchld);

    pid = fork();

    /* parent */
    if ( pid > 0 )
    {
        /* save pid 
         */
        if (pid_ptr!=NULL)
            *pid_ptr = pid;
        return 0;
    }

    /* error */
    if (pid < 0 )
    {
        signal( SIGCHLD, SIG_IGN);
        return 0;
    }
    /* child */
    if (pid == 0)
    {
        char *argv[4];
        extern char **environ;
        int ret=0;

        argv[0] = "sh";
        argv[1] = "-c";
        argv[2] = (char *)command;
        argv[3] = NULL;
        ret = execve("/bin/sh", argv, environ);
        exit(ret);
    }

    return 0;

}


int
show_yes_no_dialog (GtkWindow *main_window, char *title, char *message)
{
    GtkWidget *dialog, *label;
    int button;

    dialog = gnome_dialog_new (
        title,
        GNOME_STOCK_BUTTON_YES,
        GNOME_STOCK_BUTTON_NO,
        NULL);
//  gtk_window_set_position( GNOME_DIALOG(dialog)->window, GTK_WIN_POS_CENTER); 

    if (main_window)
       gnome_dialog_set_parent (GNOME_DIALOG (dialog), main_window);

    label = gtk_label_new (message);
    gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label, TRUE, TRUE, 0);
    gtk_widget_show(label);
    
    /*
     * Run the dialog and wait for the user to select yes or no.
     * If the user closes the window with the window manager, we
     * will get a -1 return value
     */
    button = gnome_dialog_run_and_close (GNOME_DIALOG (dialog));

    return button;
}

