/*
 * yopytodayconf 
 *
 * Choi Jonghwan <saxboy@freechal.com>
 * 2002/04/10
 */

#include <stdio.h>
#include <dlfcn.h>
#include <dirent.h>
#include <sys/types.h>
#include <gnome.h>
#include "yopytoday_resource.h"
#include "ytplugin.h"
#include "util.h"


typedef struct __YOPYTODAYCONF {

	GtkWidget *window; 
		GtkWidget *vbox; 
			GtkWidget *button_hbox; 
				GtkWidget *button_up; 
				GtkWidget *button_down; 
				GtkWidget *button_enable; 
				GtkWidget *button_disable; 
			GtkWidget *plugin_clist; 
			GtkWidget *plugin_desc_clist; 

	GList *plugin_list; 

} _YOPYTODAYCONF; 
typedef _YOPYTODAYCONF *YOPYTODAYCONF; 

static int _yopytodayconf_create_interface(YOPYTODAYCONF yt); 


YOPYTODAYCONF yopytodayconf_New(void)
{
	YOPYTODAYCONF yt=NULL; 

	if ( (yt=calloc(1, sizeof(_YOPYTODAYCONF) ))==NULL) 
		return NULL; 

	if ( _yopytodayconf_create_interface(yt) < 0 )
		goto return_error; 

	return yt; 

return_error:
	if (yt!=NULL) free(yt); 
	return NULL; 
}

int yopytodayconf_Delete(YOPYTODAYCONF ytcfg)
{
	if (ytcfg==NULL) return -1; 

	free(ytcfg); 

	return 1; 
}

int yopytodayconf_RestartYopyToday(YOPYTODAYCONF ytcfg)
{
	pid_t childpid; 

	if (ytcfg==NULL) return -1; 

	/* kill yopytoday */
	system("killall yopytoday"); 

	/* restart yopytoday */
	execute_command_background("yopytoday", &childpid); 

	return 1; 
}

int yopytodayconf_SaveConfiguration(YOPYTODAYCONF ytcfg)
{
	int item_count=0; 
	int i=0; 
	FILE *fp=NULL; 

	if (ytcfg==NULL) return -1; 


	if ( (fp=fopen( YT_CONFIG_FILE_NAME, "w+") )==NULL)
	{
		return -1; 
	}

	/* for each item in the columned list, save the item 
	 */
	item_count = GTK_CLIST(ytcfg->plugin_clist)->rows; 
	for (i=0; i<item_count; i++)
	{
		char *cell0_text=NULL; 
		char *cell1_text=NULL; 
		YT_PLUGIN plugin=NULL; 

		gtk_clist_get_text(  GTK_CLIST(ytcfg->plugin_clist), i, 0, &cell0_text); 
		gtk_clist_get_text(  GTK_CLIST(ytcfg->plugin_clist), i, 1, &cell1_text); 
		plugin = (YT_PLUGIN) gtk_clist_get_row_data( 
							GTK_CLIST(ytcfg->plugin_clist), i); 


		//printf("[%d] %s:%s:%s\n", i, cell0_text, cell1_text, plugin->path); 
		if ( cell0_text[0]== 'o' )
		{
			fprintf(fp, "%s\n", plugin->path); 
		}
		else 
		{
			fprintf(fp, "#%s\n", plugin->path); 
		}
	}


	fclose(fp); 

	printf("Yopytoday Configuration was saved to [%s]\n",YT_CONFIG_FILE_NAME);
	return 1; 
}

static void _yopytodayconf_destroy_cb(
		GtkWidget *widget, GdkEvent *e, gpointer user_data)
{
	YOPYTODAYCONF ytcfg = (YOPYTODAYCONF)(user_data); 
	int retval=0; 

	/* check if the user wants to save the result 
	 */
	retval = show_yes_no_dialog( 
				GTK_WINDOW(ytcfg->window), "",  " ұ");
	switch ( retval )
	{
		case 0 : /* YES */
			yopytodayconf_SaveConfiguration(ytcfg); 
			yopytodayconf_RestartYopyToday(ytcfg); 
			break;
		case 1 : /* NO */
		case -1 : /* CANCEL */
		default:
			break;
	}

	yopytodayconf_UnregisterPluginAll(ytcfg); 
	yopytodayconf_Delete(ytcfg); 
	gtk_main_quit(); 

	exit(1); 
}

static int g_cur_row; 

static void _yopytodayconf_plugin_clist_selected_cb(
	GtkCList *clist, int row, int column, 
	GdkEventButton *event, gpointer user_data)
{
	YOPYTODAYCONF ytcfg = (YOPYTODAYCONF)user_data; 
	YT_PLUGIN plugin=NULL; 

	if (ytcfg==NULL) return; 

	g_cur_row = row; 
	//printf("cur_row = %d\n", g_cur_row); 

	plugin = (YT_PLUGIN) gtk_clist_get_row_data( 
							GTK_CLIST(ytcfg->plugin_clist), row); 

	//printf("plugin [%s], %s, %s, %s, %s, %s\n", plugin->name, plugin->version, plugin->author, plugin->date, plugin->iconpath, plugin->path); 

#if 0
	gtk_clist_clear( GTK_CLIST(ytcfg->plugin_desc_clist) ); 
	gtk_clist_set_text( GTK_CLIST(ytcfg->plugin_desc_clist), 0, 0, "Name"); 
#endif



}
static void _yopytodayconf_button_disable_clicked_cb(
	GtkWidget *widget, gpointer user_data)
{
	YOPYTODAYCONF ytcfg=(YOPYTODAYCONF)user_data; 

	if (ytcfg==NULL) return; 

	gtk_clist_set_text( GTK_CLIST(ytcfg->plugin_clist), g_cur_row, 0, "x "); 

}
static void _yopytodayconf_button_enable_clicked_cb(
	GtkWidget *widget, gpointer user_data)
{
	YOPYTODAYCONF ytcfg=(YOPYTODAYCONF)user_data; 

	if (ytcfg==NULL) return; 
	gtk_clist_set_text( GTK_CLIST(ytcfg->plugin_clist), g_cur_row, 0, "o "); 

}
static void _yopytodayconf_button_up_clicked_cb(
	GtkWidget *widget, gpointer user_data)
{
	YOPYTODAYCONF ytcfg=(YOPYTODAYCONF)user_data; 
	int target_row=0; 

	if (ytcfg==NULL) return; 
	if (g_cur_row <= 0 ) return; 

	target_row = g_cur_row-1; 
	gtk_clist_row_move( GTK_CLIST(ytcfg->plugin_clist), g_cur_row, target_row); 
//printf("up:moved row %d to %d\n", g_cur_row, target_row); 
	g_cur_row = target_row; 

}
static void _yopytodayconf_button_down_clicked_cb(
	GtkWidget *widget, gpointer user_data)
{
	YOPYTODAYCONF ytcfg=(YOPYTODAYCONF)user_data; 
	int target_row=0; 
	int child_count=0; 

	if (ytcfg==NULL) return; 

	child_count = GTK_CLIST(ytcfg->plugin_clist)->rows; 
//printf("child_count=%d, cur_row = %d\n", child_count, g_cur_row); 
	if (g_cur_row >= child_count-1 ) 
		return; 

	target_row = g_cur_row+1; 
	gtk_clist_row_move( GTK_CLIST(ytcfg->plugin_clist), g_cur_row, target_row); 
//printf("down:moved row %d to %d\n", g_cur_row, target_row); 
	g_cur_row = target_row; 
}

static int _yopytodayconf_create_interface(YOPYTODAYCONF ytcfg)
{
	if (ytcfg==NULL) return -1; 

	ytcfg->window = gtk_window_new(GTK_WINDOW_TOPLEVEL); 
	gtk_widget_set_usize( ytcfg->window, SCREEN_WIDTH, SCREEN_HEIGHT); 

	ytcfg->vbox = gtk_vbox_new(FALSE, 0); 

	gtk_container_add(GTK_CONTAINER(ytcfg->window), ytcfg->vbox); 

	/* add button box
	 */
	ytcfg->button_hbox = gtk_hbox_new(TRUE, 0 ); 
	ytcfg->button_up = gtk_button_new_with_imagefile( YTCFG_PIC_UP ); 
	ytcfg->button_down = gtk_button_new_with_imagefile( YTCFG_PIC_DOWN ); 
	ytcfg->button_enable = gtk_button_new_with_imagefile( YTCFG_PIC_ENABLE); 
	ytcfg->button_disable = gtk_button_new_with_imagefile( YTCFG_PIC_DISABLE); 
	gtk_box_pack_start( 
		GTK_BOX(ytcfg->button_hbox), ytcfg->button_up, TRUE, TRUE, 1 ); 
	gtk_box_pack_start( 
		GTK_BOX(ytcfg->button_hbox), ytcfg->button_down, TRUE, TRUE, 1 ); 
	gtk_box_pack_start( 
		GTK_BOX(ytcfg->button_hbox), ytcfg->button_enable, TRUE, TRUE, 1 ); 
	gtk_box_pack_start( 
		GTK_BOX(ytcfg->button_hbox), ytcfg->button_disable, TRUE, TRUE, 1 ); 

	gtk_signal_connect(
		GTK_OBJECT(ytcfg->button_up), "clicked", 
		(GtkSignalFunc) _yopytodayconf_button_up_clicked_cb, 
		(gpointer) ytcfg); 
	gtk_signal_connect(
		GTK_OBJECT(ytcfg->button_down), "clicked", 
		(GtkSignalFunc) _yopytodayconf_button_down_clicked_cb, 
		(gpointer) ytcfg); 
	gtk_signal_connect(
		GTK_OBJECT(ytcfg->button_enable), "clicked", 
		(GtkSignalFunc) _yopytodayconf_button_enable_clicked_cb, 
		(gpointer) ytcfg); 
	gtk_signal_connect(
		GTK_OBJECT(ytcfg->button_disable), "clicked", 
		(GtkSignalFunc) _yopytodayconf_button_disable_clicked_cb, 
		(gpointer) ytcfg); 


	ytcfg->plugin_clist = gtk_clist_new(2); 
	gtk_clist_set_column_auto_resize( GTK_CLIST(ytcfg->plugin_clist), 0, TRUE); 
	gtk_clist_set_column_auto_resize( GTK_CLIST(ytcfg->plugin_clist), 1, TRUE); 
	gtk_signal_connect(
		GTK_OBJECT(ytcfg->plugin_clist), "select-row", 
		(GtkSignalFunc) _yopytodayconf_plugin_clist_selected_cb, 
		(gpointer) ytcfg); 




	/* append logo 
	 */
	gtk_box_pack_start( 
		GTK_BOX(ytcfg->vbox), 
		get_picture_pixmap( YT_PIC_LOGO), 
		FALSE, FALSE, 1 ); 
	gtk_box_pack_start( 
		GTK_BOX(ytcfg->vbox), 
		ytcfg->button_hbox, 
		FALSE, FALSE, 1 ); 
	gtk_box_pack_start( 
		GTK_BOX(ytcfg->vbox), 
		ytcfg->plugin_clist, 
		TRUE, TRUE, 1 ); 

	/* FIXME: show plugin info in desc clist
	 */
#if 0
	ytcfg->plugin_desc_clist = gtk_clist_new(1); 
	gtk_box_pack_start( 
		GTK_BOX(ytcfg->vbox), 
		ytcfg->plugin_desc_clist, 
		TRUE, TRUE, 1 ); 
#endif

	/* destroy callback
	 */
	gtk_signal_connect( 
		GTK_OBJECT(ytcfg->window), 
		"delete_event", 
		(GtkSignalFunc) _yopytodayconf_destroy_cb, 
		(gpointer)ytcfg); 

	return 1; 
}

int yopytodayconf_RegisterPlugin(YOPYTODAYCONF yt, const char *plugin_path)
{
	void *libhandle=NULL; 
	YT_PLUGIN plugin=NULL; 
	YT_PLUGIN (*infofunc)(void) =NULL; 

	if (yt==NULL) return -1; 
	if (plugin_path==NULL) return -1; 

	/* load given plungin 
	 */
	if ( (libhandle = dlopen(plugin_path, RTLD_NOW))==NULL)
		return -1; 

	/* get plugin info func
	 */
	if ( (infofunc = dlsym( libhandle, "yt_plugin_get_info") ) == NULL)
		goto return_error; 

	/* get plugin object from the info func
	 */	
	if ( (plugin = (*infofunc)())==NULL)
		goto return_error; 

	plugin->path = strdup(plugin_path); 

	/* add this plugin to the plugin clist
	 */
	{
		char *textarr[2]={NULL,}; 
		int cur_row=0; 

		textarr[0] = strdup("x "); 
		textarr[1] = strdup(plugin->name); 

		cur_row = gtk_clist_append(GTK_CLIST(yt->plugin_clist), textarr); 
		gtk_clist_set_row_data( 
			GTK_CLIST(yt->plugin_clist), cur_row, (gpointer)plugin); 

		free(textarr[0]); 
		free(textarr[1]); 
	}

	/* register the plugin itself
	 */
	yt->plugin_list = g_list_append(yt->plugin_list, plugin ); 

	return 1; 

return_error:
	if (libhandle!=NULL) dlclose(libhandle); 
	return -1; 
}

int yopytodayconf_UnregisterPluginAll(YOPYTODAYCONF yt)
{
	GList *l=NULL; 

	if ( yt==NULL) return -1; 

	/* for each plugin in the list, finalize it
	 */
	for (l=yt->plugin_list; l!=NULL; l=l->next)
	{
		YT_PLUGIN plugin = (YT_PLUGIN)(l->data); 
		if (plugin->path!=NULL) free(plugin->path); 
		//(*plugin->yt_plugin_finalize)(); 

//printf("Unregistered plugin [%s]\n", plugin->name); 
	}

	return 1; 	
}

int yopytodayconf_ScanPlugin(YOPYTODAYCONF yt, const char *dir)
{
	DIR *dp; 
	struct dirent *dirp; 

	if (yt==NULL) return -1; 
	if (dir==NULL) return -1; 

	if ( (dp=opendir(dir))==NULL)
	{
		return -1; 
	}

	while ( (dirp=readdir(dp))!=NULL) 
	{
		char filename[1024]={0,}; 
		if ( strcmp( dirp->d_name, ".") == 0 )
			continue; 
		if ( strcmp( dirp->d_name, "..") == 0 )
			continue; 

		sprintf(filename, "%s/%s", dir, dirp->d_name); 
		if ( yopytodayconf_RegisterPlugin(yt, filename) > 0 )
		{
			//printf("registered [%s]\n", filename ); 
		}
	}



	return 1; 
}

int yopytodayconf_Show(YOPYTODAYCONF yt)
{
	if (yt==NULL) return -1; 

	gtk_widget_show_all(yt->window); 

	return 1; 
}


int main(int argc, char *argv[])
{
	YOPYTODAYCONF yt=NULL; 

	gtk_set_locale(); 
	gtk_rc_add_default_file("./yopytodayconf.rc"); 
	gnome_init(PROG_NAME, PROG_VERSION, argc, argv); 

	/* run yopytodayconf
	 */
	if ((yt = yopytodayconf_New())==NULL) return -1; 
	yopytodayconf_ScanPlugin(yt, YT_PLUGIN_DIR ); 
	yopytodayconf_Show(yt); 

	gtk_main(); 
	return 0; 
}
