/*
 * memory plugin for yopytoday
 *
 * Choi Jonghwan <saxboy@freechal.com>
 * 2003/04/23
 */

#include <stdio.h>
#include "ytplugin.h"

static void yt_plugin_battery_initialize(void); 
static void yt_plugin_battery_finalize(void); 
static void yt_plugin_battery_update(void); 
static GtkWidget *yt_plugin_battery_get_window(void); 


_YT_PLUGIN g_yt_plugin_memory = 
{
	"yopytoday_battery_plugin", 
	"0.01", 
	"Choi Jonghwan", 
	"http://saxboy.wo.to", 
	"saxboy@freechal.com", 
	__DATE__, 
	"yt_plugin_battery.png", 
	1000, 
	210, 
	20, 
	yt_plugin_battery_initialize, 
	yt_plugin_battery_finalize, 
	yt_plugin_battery_update, 
	yt_plugin_battery_get_window, 
	NULL, 

	/* internal */
	NULL
}; 


YT_PLUGIN yt_plugin_get_info(void)
{
	return &g_yt_plugin_memory; 
}


/* -------------------------------------------------------------
 * internal plugin routines. 
 */
static GtkWidget *ytp_battery_window; 
static GtkWidget *_battery_progress; 
static GtkWidget *_battery_label; 

static void yt_plugin_battery_initialize(void)
{
	ytp_battery_window = gtk_table_new(1,3, TRUE); 

	/* create progress window 
	 */
	_battery_progress = gtk_progress_bar_new(); 
	gtk_progress_set_value( GTK_PROGRESS(_battery_progress), (gfloat)(0.0) ); 
	gtk_progress_set_show_text( GTK_PROGRESS(_battery_progress), TRUE ); 
	gtk_progress_set_format_string( GTK_PROGRESS(_battery_progress), "%p%%"); 

	/* create label
	 */
	_battery_label = gtk_label_new(NULL); 

	/* attach widgets to the table
	 */
	gtk_table_attach( 
		GTK_TABLE(ytp_battery_window), 
		_battery_progress, 
		0, 2, 0, 1, 
		GTK_FILL| GTK_EXPAND| GTK_SHRINK, GTK_FILL| GTK_EXPAND| GTK_SHRINK, 
		0, 0 ); 
	gtk_table_attach( 
		GTK_TABLE(ytp_battery_window), 
		_battery_label, 
		2, 3, 0, 1, 
		GTK_FILL| GTK_EXPAND| GTK_SHRINK, GTK_FILL| GTK_EXPAND| GTK_SHRINK, 
		0, 0 ); 

	gtk_widget_show_all(ytp_battery_window); 

	yt_plugin_battery_update(); 
}

static void yt_plugin_battery_finalize(void)
{
	gtk_widget_hide_all(ytp_battery_window); 
}


#ifdef YOPY

#include <yopylib.h>

static void yt_plugin_battery_update_real(void)
{
	yopy_power_status_t power_status; 
	float battery_life=0.0; 

	/* get power info 
	 */
	yopy_power_status( &power_status); 

	/* update battery/ac icon
	 */

	/* determine battery life
	 */
	battery_life = (float)(power_status.battery_life_time); 

	/* set battery progress value 
	 */
	gtk_progress_configure( GTK_PROGRESS(_battery_progress),0,0,100); 

	gtk_progress_set_value( 
		GTK_PROGRESS( _battery_progress), 
		(float)(power_status.battery_life_percent) ); 

	/* set text */
	{
		char labeltext[1024]={0,}; 
		sprintf(labeltext, "%3.2f Min", battery_life ); 
		gtk_label_set_text( GTK_LABEL( _battery_label), labeltext); 
	}

	return; 
}

#endif

static void yt_plugin_battery_update_pseudo(void)
{
	float battery_life=0.0; 

	/* update battery/ac icon
	 */

	/* determine battery life
	 */
	battery_life=10; 

	/* set battery progress value 
	 */
	gtk_progress_configure( GTK_PROGRESS(_battery_progress),0,0,100); 

	gtk_progress_set_value( 
		GTK_PROGRESS( _battery_progress), 45 ); 

	/* set text */
	{
		char labeltext[1024]={0,}; 
		sprintf(labeltext, "%3.2f Min", battery_life ); 
		gtk_label_set_text( GTK_LABEL( _battery_label), labeltext); 
	}

	return; 
}


static void yt_plugin_battery_update(void)
{
#ifdef YOPY 
	yt_plugin_battery_update_real(); 
#else
	yt_plugin_battery_update_pseudo(); 
#endif

}

static GtkWidget *yt_plugin_battery_get_window(void)
{
	return ytp_battery_window; 
}



