/*
 * sample plugin for yopytoday
 *
 * Choi Jonghwan <saxboy@freechal.com>
 * 2003/04/11
 */

#include <stdio.h>
#include <mysql.h>
#include "yopytoday_resource.h"
#include "ytplugin.h"
#include "util.h"

static void yt_plugin_calendar_initialize(void); 
static void yt_plugin_calendar_finalize(void); 
static void yt_plugin_calendar_update(void); 
static GtkWidget *yt_plugin_calendar_get_window(void); 
static GtkWidget *yt_plugin_calendar_get_config_window(void); 


_YT_PLUGIN g_yt_plugin_calendar = 
{
	"yopytoday_calendar_plugin", 
	"0.01", 
	"Choi Jonghwan", 
	"http://saxboy.wo.to", 
	"saxboy@freechal.com", 
	__DATE__, 
	"yt_plugin_calendar.png", 
	10000, 	/* update delay */
	210, 	/* x res req */
	80,		/* y res req */ 
	yt_plugin_calendar_initialize, 
	yt_plugin_calendar_finalize, 
	yt_plugin_calendar_update, 
	yt_plugin_calendar_get_window, 
	yt_plugin_calendar_get_config_window, 

	/* internal */
	NULL
}; 


YT_PLUGIN yt_plugin_get_info(void)
{
	return &g_yt_plugin_calendar; 
}


/* -------------------------------------------------------------
 * internal plugin routines. 
 */

static GtkWidget *ytp_calendar_window; 
static GtkWidget *ytp_calendar_clist; 
static MYSQL ytp_calendar_mysql; 

static char ytp_calendar_query[1024]; 

#define YTP_CALENDAR_CONFIG		YT_PLUGIN_DIR "/yt_plugin_calendar.query"

/* FIXME: 
 * this query will not work if today is the last day of the month. 
 */
#define YTP_CALENDAR_QUERY_DEFAULT  "select startTime, subject from Schedule where startTime >= CURRENT_DATE() and startTime < CURRENT_DATE()+1 "

static void  _calendar_append_entry(const char *time, const char *text)
{
	char *textarr[2] = {NULL, }; 

//printf("append calendar [%s:%s]\n", time, text); 
	if ( time!=NULL) textarr[0] = strdup(&time[11]); 
	if ( text!=NULL) textarr[1] = strdup(text); 
	gtk_clist_append( GTK_CLIST(ytp_calendar_clist), textarr); 
}

static void yt_plugin_calendar_initialize(void)
{
	/* create plugin widgets
	 */
	ytp_calendar_window = gtk_scrolled_window_new(NULL, NULL); 
	gtk_scrolled_window_set_policy( 
		GTK_SCROLLED_WINDOW( ytp_calendar_window), 
		GTK_POLICY_NEVER, 
		GTK_POLICY_AUTOMATIC); 

	ytp_calendar_clist = gtk_clist_new(2); 

	/* read todo items from mysql & append it to the clist
	 */
#if 0
	gtk_clist_set_column_title( GTK_CLIST(ytp_calendar_clist), 0, "priority" ); 
	gtk_clist_set_column_title( GTK_CLIST(ytp_calendar_clist), 1, "todo" ); 
	gtk_clist_column_titles_show( GTK_CLIST(ytp_calendar_clist)); 
#endif
	gtk_clist_set_column_auto_resize( GTK_CLIST(ytp_calendar_clist), 0, TRUE); 
	gtk_clist_set_column_auto_resize( GTK_CLIST(ytp_calendar_clist), 1, TRUE); 


	gtk_container_add( 
		GTK_CONTAINER(ytp_calendar_window), ytp_calendar_clist); 	
	gtk_widget_show_all(ytp_calendar_window); 

	/* connect to the database first
	 */
	if ( mysql_init( &ytp_calendar_mysql) == NULL )
		return; 
	if ( !mysql_real_connect( &ytp_calendar_mysql, "localhost", "root", "", "YopimDB", 3306, NULL, 0 )) 
		return; 

	/* initialize customizable query
	 */
    {
		FILE *fp=NULL;
		if ( (fp=fopen(YTP_CALENDAR_CONFIG, "r"))!=NULL)
		{
			fgets( ytp_calendar_query, 1024, fp);
			fclose(fp); 
		}
	}
	if ( ytp_calendar_query[0]=='\0' )
		strncpy( ytp_calendar_query, YTP_CALENDAR_QUERY_DEFAULT, 1024);

	printf("calendar: initailized query [%s]\n", ytp_calendar_query); 

	/* update it first
	 */
	yt_plugin_calendar_update(); 
}

static void yt_plugin_calendar_finalize(void)
{
	/* destroy widgets
	 */
	gtk_widget_hide_all(ytp_calendar_window); 
	
	/* disconnect from the database
	 */
	mysql_close( &ytp_calendar_mysql); 
}

static void yt_plugin_calendar_update(void)
{
//	printf("update calendar... \n"); 
	/* send query to the database
	 */
	if ( mysql_query( &ytp_calendar_mysql, ytp_calendar_query ) )
		return; 

	/* refresh the results
	 */
	gtk_clist_clear( GTK_CLIST(ytp_calendar_clist) ); 

    {
        MYSQL_RES *res=NULL;
        MYSQL_ROW row;
        int field_cnt=0;
        int row_cnt=0;

        res = mysql_store_result( &ytp_calendar_mysql);
        field_cnt = mysql_num_fields(res); /* should be 2 */
        row_cnt = mysql_num_rows(res);

        while ( (row=mysql_fetch_row(res)) )
        {
            _calendar_append_entry( row[0], row[1] );
        }
    }

#if 0
	/* force redraw 
	 */
	gtk_widget_draw( ytp_calendar_window, NULL); 
#endif

}

static GtkWidget *yt_plugin_calendar_get_window(void)
{
	return ytp_calendar_window; 
}

/* -------------------------------------------------------
 * configuration window
 *
 */

static GtkWidget *ytp_calendar_config_window; 

static 	GtkWidget *config_vbox; 
static 		GtkWidget *button_hbox; 
static 			GtkWidget *button_default; 
static 			GtkWidget *button_save; 
static 		GtkWidget *query_text; 


static void _config_default_clicked_cb( GtkWidget *widget, gpointer data)
{
	int pos=0; 

	gtk_editable_delete_text(GTK_EDITABLE(query_text), 0, -1); 
	gtk_editable_insert_text( 
		GTK_EDITABLE(query_text), 
			YTP_CALENDAR_QUERY_DEFAULT, strlen(YTP_CALENDAR_QUERY_DEFAULT), &pos); 
}

static void _config_save_clicked_cb( GtkWidget *widget, gpointer data)
{
    FILE *fp=NULL;
	gchar *query_string = NULL; 

	query_string = gtk_editable_get_chars( GTK_EDITABLE(query_text), 0, -1); 
	if ( query_string==NULL)
	{
    	show_message("˼  ߻Ͽϴ."); 
		return; 
	}

	if ( (fp=fopen(YTP_CALENDAR_CONFIG, "w+"))==NULL)
	{
    	show_message("   ϴ"); 
        return; 
	}

	fprintf(fp, "%s\n", query_string); 

	fclose(fp);


    show_message(" Ǿϴ.\nyopytoday ٽ ϼ");
}

static GtkWidget *yt_plugin_calendar_get_config_window(void)
{
	config_vbox = gtk_vbox_new(FALSE, 0); 

	button_hbox = gtk_hbox_new(TRUE, 1 ); 

//show_message("1"); 
	button_default = gtk_button_new_with_imagefile( YT_RESOURCE_DIR "ytp_calendar_default.png"); 

	button_save = gtk_button_new_with_imagefile( YT_RESOURCE_DIR "ytp_calendar_save.png"); 
printf("loading [%s]\n", YT_RESOURCE_DIR "/ytp_calendar_default.png"); 
printf("loading [%s]\n", YT_RESOURCE_DIR "/ytp_calendar_save.png"); 

//abort(); 
if (button_default==NULL || button_save==NULL) 	
	printf("button is NULL!!!\n"); 

//abort(); 
	gtk_signal_connect(
		GTK_OBJECT(button_default), "clicked", 
		(GtkSignalFunc) _config_default_clicked_cb, NULL); 
	gtk_signal_connect(
		GTK_OBJECT(button_save), "clicked", 
		(GtkSignalFunc) _config_save_clicked_cb, NULL); 

//abort(); 
	gtk_box_pack_start(GTK_BOX(button_hbox), button_default, TRUE, TRUE, 0 ); 
	gtk_box_pack_start(GTK_BOX(button_hbox), button_save, TRUE, TRUE, 0 ); 

	query_text = gtk_text_new(NULL, NULL); 
	gtk_text_set_editable( GTK_TEXT(query_text), TRUE); 
	gtk_text_set_word_wrap( GTK_TEXT(query_text), TRUE); 

	gtk_box_pack_start(GTK_BOX(config_vbox), button_hbox, FALSE, FALSE, 0 ); 
	gtk_box_pack_start(GTK_BOX(config_vbox), query_text, TRUE, TRUE, 0 ); 
//abort(); 

	/* set default configuraion
	 */
	{
		FILE *fp=NULL; 
		char query_string[1024]={0,}; 
		int pos=0; 

		if ( (fp=fopen(YTP_CALENDAR_CONFIG, "r"))!=NULL)
		{
			fgets( query_string, 1024, fp); 
			fclose(fp); 
		}
		if ( query_string[0]=='\0' )
			strncpy( query_string, YTP_CALENDAR_QUERY_DEFAULT, 1024); 

		gtk_editable_insert_text( 
			GTK_EDITABLE(query_text), 
			query_string, strlen(query_string), &pos); 
	}


	ytp_calendar_config_window = config_vbox; 
	return ytp_calendar_config_window; 
}
