/*
 * sample plugin for yopytoday
 *
 * Choi Jonghwan <saxboy@freechal.com>
 * 2003/04/11
 */

#include <stdio.h>
#include <mysql.h>
#include "ytplugin.h"
#include "yopytoday_resource.h"
#include "util.h"

static void yt_plugin_clock_initialize(void); 
static void yt_plugin_clock_finalize(void); 
static void yt_plugin_clock_update(void); 
static GtkWidget *yt_plugin_clock_get_window(void); 


_YT_PLUGIN g_yt_plugin_clock = 
{
	"yopytoday_clock_plugin", 
	"0.01", 
	"Choi Jonghwan", 
	"http://saxboy.wo.to", 
	"saxboy@freechal.com", 
	__DATE__, 
	"yt_plugin_clock.png", 
	10000, 
	210, 
	24, 
	yt_plugin_clock_initialize, 
	yt_plugin_clock_finalize, 
	yt_plugin_clock_update, 
	yt_plugin_clock_get_window, 
	NULL, 

	/* internal */
	NULL
}; 


YT_PLUGIN yt_plugin_get_info(void)
{
	return &g_yt_plugin_clock; 
}


/* -------------------------------------------------------------
 * internal plugin routines. 
 */
#define YTP_CLOCK_DEFAULT_PATH	"xclock"
#define YTP_CLOCK_CONF_FILE		YT_PLUGIN_DIR "/yt_plugin_clock.conf"

static GtkWidget *ytp_clock_window; 
static GtkWidget *_ytp_clocklabel=NULL; 
static char _ytp_clock_path[1024]; 


static void _yt_plugin_clock_clicked_cb(
				GtkWidget *widget, 
				gpointer data)
{
	pid_t childpid=0; 

	printf("clicked: execute [%s]\n", _ytp_clock_path); 

	/* run extern clock application
	 */
	execute_command_background( _ytp_clock_path, &childpid); 
	
}

static void yt_plugin_clock_initialize(void)
{
	/* initialize external clock program 
	 */
	if ( file_Exist( YTP_CLOCK_CONF_FILE ) )
	{
		FILE *fp=NULL; 
		if ( (fp=fopen(YTP_CLOCK_CONF_FILE, "r"))!=NULL)
		{
			char buf[1024]={0,}; 
			if ( fgets(buf, 1024, fp)!=NULL)
			{
				if (buf[0]!='\0' )
				{
					if ( buf[strlen(buf)-1] == '\n' )
						buf[strlen(buf)-1] = '\0'; 
					strncpy( _ytp_clock_path, buf, 1024); 
				}
			}
		}
	}

	if ( _ytp_clock_path[0]=='\0' )
		strncpy( _ytp_clock_path, YTP_CLOCK_DEFAULT_PATH, 1024); 

	/* create plugin widgets
	 */
	ytp_clock_window = gtk_event_box_new(); 
	_ytp_clocklabel = gtk_label_new("?"); 
	gtk_container_add( GTK_CONTAINER(ytp_clock_window), _ytp_clocklabel); 

	gtk_signal_connect(
		GTK_OBJECT(ytp_clock_window), 
		"button-press-event", 
		(GtkSignalFunc) _yt_plugin_clock_clicked_cb, 
		NULL); 

	/* some decorations
	 */
	//gdk_window_set_background( ytp_clock_window->window, color); 

	gtk_widget_show_all(ytp_clock_window); 

	yt_plugin_clock_update(); 
}

static void yt_plugin_clock_finalize(void)
{
	//gtk_widget_hide_all(ytp_clock_window); 
}

static void yt_plugin_clock_update(void)
{
	//char today_text[128]={0,}; 
	char label_text[128]={0,}; 
	char wday_string[8]={0,}; 
	time_t cur_time; 
	struct tm cur_localtime; 


	time( &cur_time); 
	localtime_r( &cur_time, &cur_localtime); 

	switch ( cur_localtime.tm_wday )
	{
		case 0: strcpy(wday_string, ""); break;
		case 1: strcpy(wday_string, ""); break;
		case 2: strcpy(wday_string, "ȭ"); break;
		case 3: strcpy(wday_string, ""); break;
		case 4: strcpy(wday_string, ""); break;
		case 5: strcpy(wday_string, ""); break;
		case 6: strcpy(wday_string, ""); break;
		default: strcpy(wday_string, "??"); break;
	}

	/* FIXME: add more options to time representation
	 */
	sprintf(label_text, "%d  %d  %d  (%s) : %d  %d ",
		 cur_localtime.tm_year+1900, 
		 cur_localtime.tm_mon+1, 
		 cur_localtime.tm_mday, 
		 wday_string, 
		 cur_localtime.tm_hour, 
		 cur_localtime.tm_min ); 
	
	//ctime_r( &cur_time, today_text); 
	//sprintf(label_text, "\n%s", today_text); 

	//printf("update clock [%s]\n", today_text); 
	gtk_label_set_text( GTK_LABEL( _ytp_clocklabel) , label_text); 

	/* force redraw
	 */
	gtk_widget_draw( ytp_clock_window, NULL); 
}

static GtkWidget *yt_plugin_clock_get_window(void)
{
	return ytp_clock_window; 
}

