/*
 * sample plugin for yopytoday
 *
 * Choi Jonghwan <saxboy@freechal.com>
 * 2003/04/11
 */

#include <stdio.h>
#include "ytplugin.h"
#include "yopytoday_resource.h"
#include "util.h"

static void yt_plugin_dict_initialize(void); 
static void yt_plugin_dict_finalize(void); 
static GtkWidget *yt_plugin_dict_get_window(void); 
static GtkWidget *yt_plugin_dict_get_config_window(void); 


_YT_PLUGIN g_yt_plugin_dict = 
{
	"yopytoday_dict_plugin", 
	"0.01", 
	"Choi Jonghwan", 
	"http://saxboy.wo.to", 
	"saxboy@freechal.com", 
	__DATE__, 
	"yt_plugin_dict.png", 
	0, 
	210, 
	20, 
	yt_plugin_dict_initialize, 
	yt_plugin_dict_finalize, 
	NULL, 
	yt_plugin_dict_get_window, 
	yt_plugin_dict_get_config_window, 

	/* internal */
	NULL
}; 


YT_PLUGIN yt_plugin_get_info(void)
{
	return &g_yt_plugin_dict; 
}


/* -------------------------------------------------------------
 * internal plugin routines. 
 */
#define YTP_DICT_DEFAULT_PATH	"gtkdic -w "
#define YTP_DICT_CONFIG		YT_PLUGIN_DIR "/yt_plugin_dict.conf"

static GtkWidget *ytp_dict_window; 
static GtkWidget *_word_entry; 
static GtkWidget *_submit_button; 
static char _ytp_dict_path[1024]; 

static void _yt_plugin_dict_button_clicked_cb( 
				GtkWidget *widget, gpointer data)
{
	gchar *word=NULL; 
	char command_buffer[1024]={0,}; 
	pid_t child_pid; 

	word = gtk_entry_get_text( GTK_ENTRY(_word_entry) ); 

	sprintf(command_buffer, "%s \"%s\"", _ytp_dict_path, word); 
//	printf("command=[%s]\n", command_buffer); 

	execute_command_background( command_buffer, &child_pid); 
}

static void yt_plugin_dict_initialize(void)
{

	/* initialize external dictionary path
	 */
	if ( file_Exist( YTP_DICT_CONFIG ) )
	{
		FILE *fp=NULL; 
		if ( (fp = fopen( YTP_DICT_CONFIG, "r"))!=NULL)
		{
			char buf[1024]={0,}; 
			if ( fgets(buf, 1024, fp)!=NULL) 
			{
				if ( buf[0]!='\0' )
				{
					if ( buf[strlen(buf)-1]=='\n' )
						buf[strlen(buf)-1] = '\0'; 
					strncpy( _ytp_dict_path, buf, 1024); 
				}
			}

		}
	} 
	
	if ( _ytp_dict_path[0]=='\0' ) 
		strncpy(_ytp_dict_path, YTP_DICT_DEFAULT_PATH, 1024); 

	/* create dictionary plugin widgets
	 */
	ytp_dict_window = gtk_hbox_new(FALSE, 0 ); 

	_word_entry = gtk_entry_new_with_max_length(15); 
	_submit_button = gtk_button_new_with_label("Search"); 

	gtk_box_pack_start( GTK_BOX(ytp_dict_window), _word_entry, TRUE, TRUE, 0); 
	gtk_box_pack_start( GTK_BOX(ytp_dict_window), _submit_button, FALSE, FALSE, 2); 

	gtk_signal_connect( 
		GTK_OBJECT( _submit_button), "clicked", 
		(GtkSignalFunc) _yt_plugin_dict_button_clicked_cb, 
		NULL); 

	gtk_widget_show_all(ytp_dict_window); 

}

static void yt_plugin_dict_finalize(void)
{
	gtk_widget_hide_all(ytp_dict_window); 
}

static GtkWidget *yt_plugin_dict_get_window(void)
{
	return ytp_dict_window; 
}


/* -------------------------------------------------------
 * configuration window
 *
 */

static GtkWidget *ytp_dict_config_window; 

static 	GtkWidget *config_vbox; 
static 		GtkWidget *button_hbox; 
static 			GtkWidget *button_default; 
static 			GtkWidget *button_save; 
static 		GtkWidget *query_text; 


static void _config_default_clicked_cb( GtkWidget *widget, gpointer data)
{
	int pos=0; 

	gtk_editable_delete_text(GTK_EDITABLE(query_text), 0, -1); 
	gtk_editable_insert_text( 
		GTK_EDITABLE(query_text), 
			YTP_DICT_DEFAULT_PATH, strlen(YTP_DICT_DEFAULT_PATH), &pos); 
}

static void _config_save_clicked_cb( GtkWidget *widget, gpointer data)
{
    FILE *fp=NULL;
	gchar *query_string = NULL; 

	query_string = gtk_editable_get_chars( GTK_EDITABLE(query_text), 0, -1); 
	if ( query_string==NULL)
	{
    	show_message("˼  ߻Ͽϴ."); 
		return; 
	}

	if ( (fp=fopen(YTP_DICT_CONFIG, "w+"))==NULL)
	{
    	show_message("   ϴ"); 
        return; 
	}

	fprintf(fp, "%s\n", query_string); 

	fclose(fp);


    show_message(" Ǿϴ.\nyopytoday ٽ ϼ");
}

static GtkWidget *yt_plugin_dict_get_config_window(void)
{
	config_vbox = gtk_vbox_new(FALSE, 0); 

	button_hbox = gtk_hbox_new(TRUE, 1 ); 
	button_default = gtk_button_new_with_imagefile( YT_RESOURCE_DIR "ytp_dict_default.png"); 
	button_save = gtk_button_new_with_imagefile( YT_RESOURCE_DIR "ytp_dict_save.png"); 

	gtk_signal_connect(
		GTK_OBJECT(button_default), "clicked", 
		(GtkSignalFunc) _config_default_clicked_cb, NULL); 
	gtk_signal_connect(
		GTK_OBJECT(button_save), "clicked", 
		(GtkSignalFunc) _config_save_clicked_cb, NULL); 

	gtk_box_pack_start(GTK_BOX(button_hbox), button_default, TRUE, TRUE, 0 ); 
	gtk_box_pack_start(GTK_BOX(button_hbox), button_save, TRUE, TRUE, 0 ); 

	query_text = gtk_text_new(NULL, NULL); 
	gtk_text_set_editable( GTK_TEXT(query_text), TRUE); 
	gtk_text_set_word_wrap( GTK_TEXT(query_text), TRUE); 

	gtk_box_pack_start(GTK_BOX(config_vbox), button_hbox, FALSE, FALSE, 0 ); 
	gtk_box_pack_start(GTK_BOX(config_vbox), query_text, TRUE, TRUE, 0 ); 

	/* set default configuraion
	 */
	{
		FILE *fp=NULL; 
		char query_string[1024]={0,}; 
		int pos=0; 

		if ( (fp=fopen(YTP_DICT_CONFIG, "r"))!=NULL)
		{
			fgets( query_string, 1024, fp); 
			fclose(fp); 
		}
		if ( query_string[0]=='\0' )
			strncpy( query_string, YTP_DICT_DEFAULT_PATH, 1024); 

		gtk_editable_insert_text( 
			GTK_EDITABLE(query_text), 
			query_string, strlen(query_string), &pos); 
	}


	ytp_dict_config_window = config_vbox; 
	return ytp_dict_config_window; 
}
