/*
 * memory plugin for yopytoday
 *
 * Choi Jonghwan <saxboy@freechal.com>
 * 2003/04/23
 */

#include <stdio.h>
#include "ytplugin.h"

static void yt_plugin_memory_initialize(void); 
static void yt_plugin_memory_finalize(void); 
static void yt_plugin_memory_update(void); 
static GtkWidget *yt_plugin_memory_get_window(void); 


_YT_PLUGIN g_yt_plugin_memory = 
{
	"yopytoday_memory_plugin", 
	"0.01", 
	"Choi Jonghwan", 
	"http://saxboy.wo.to", 
	"saxboy@freechal.com", 
	__DATE__, 
	"yt_plugin_memory.png", 
	1000, 
	210, 
	20, 
	yt_plugin_memory_initialize, 
	yt_plugin_memory_finalize, 
	yt_plugin_memory_update, 
	yt_plugin_memory_get_window, 
	NULL, 

	/* internal */
	NULL
}; 


YT_PLUGIN yt_plugin_get_info(void)
{
	return &g_yt_plugin_memory; 
}


/* -------------------------------------------------------------
 * internal plugin routines. 
 */
#include <sys/vfs.h>

static 
void get_fs_statstics(const char *path, long *freesize_k, long *totalsize_k)
{
    struct statfs statfsbuf;

    statfs(path, &statfsbuf); 

    //*freesize_k = statfsbuf.f_bfree; 
    *freesize_k = statfsbuf.f_bavail; 
    *totalsize_k = statfsbuf.f_blocks;
}

static GtkWidget *ytp_memory_window; 
static GtkWidget *_memory_progress; 
static GtkWidget *_memory_label; 

static void yt_plugin_memory_initialize(void)
{
	ytp_memory_window = gtk_table_new(1,3, TRUE); 

	/* create progress window 
	 */
	_memory_progress = gtk_progress_bar_new(); 
	gtk_progress_set_value( GTK_PROGRESS(_memory_progress), (gfloat)(0.0) ); 
	gtk_progress_set_show_text( GTK_PROGRESS(_memory_progress), TRUE ); 
	gtk_progress_set_format_string( GTK_PROGRESS(_memory_progress), "%p%%"); 

	/* create label
	 */
	_memory_label = gtk_label_new(NULL); 

	/* attach widgets to the table
	 */
	gtk_table_attach( 
		GTK_TABLE(ytp_memory_window), 
		_memory_progress, 
		0, 2, 0, 1, 
		GTK_FILL| GTK_EXPAND| GTK_SHRINK, GTK_FILL| GTK_EXPAND| GTK_SHRINK, 
		0, 0 ); 
	gtk_table_attach( 
		GTK_TABLE(ytp_memory_window), 
		_memory_label, 
		2, 3, 0, 1, 
		GTK_FILL| GTK_EXPAND| GTK_SHRINK, GTK_FILL| GTK_EXPAND| GTK_SHRINK, 
		0, 0 ); 

	gtk_widget_show_all(ytp_memory_window); 

	yt_plugin_memory_update(); 
}

static void yt_plugin_memory_finalize(void)
{
	gtk_widget_hide_all(ytp_memory_window); 
}

static void yt_plugin_memory_update(void)
{
	long freesize_k=0; 
	long totalsize_k=0; 
	float freesize_m=0; 
	//float usage_percent=0.0; 

#ifdef YOPY
	get_fs_statstics("/home", &freesize_k, &totalsize_k); 
#else
	get_fs_statstics("/", &freesize_k, &totalsize_k); 
#endif

	//usage_percent = (float)((float)freesize_k / (float)totalsize_k); 
	freesize_m = (float)( (float)freesize_k / 1000 ); 

	gtk_progress_configure( GTK_PROGRESS(_memory_progress),0,0,totalsize_k); 
	gtk_progress_set_value( 
		GTK_PROGRESS( _memory_progress),totalsize_k - freesize_k); 

	/* set text */
	{
		char labeltext[1024]={0,}; 
		sprintf(labeltext, "%3.2f M", freesize_m ); 
		gtk_label_set_text( GTK_LABEL( _memory_label), labeltext); 
	}

	return; 
}

static GtkWidget *yt_plugin_memory_get_window(void)
{
	return ytp_memory_window; 
}



