/*
 * sample plugin for yopytoday
 *
 * Choi Jonghwan <saxboy@freechal.com>
 * 2003/04/11
 */

#include <stdio.h>
#include <ctype.h>
#include "ytplugin.h"
#include "yopytoday_resource.h"
#include "util.h"

#define UNKNOWN_ICON_PATH	YT_RESOURCE_DIR	"/yt_plugin_quicklaunch_unknow.png"
#define YT_PLUGIN_QUICKLAUNCH_CONFIG  YT_PLUGIN_DIR "/yt_plugin_quicklaunch.conf"

static void yt_plugin_quicklaunch_initialize(void); 
static void yt_plugin_quicklaunch_finalize(void); 
static void yt_plugin_quicklaunch_update(void); 
static GtkWidget *yt_plugin_quicklaunch_get_window(void); 
static GtkWidget *yt_plugin_quicklaunch_get_config_window(void); 


_YT_PLUGIN g_yt_plugin_quicklaunch = 
{
	"yopytoday_quicklaunch_plugin", 
	"0.01", 
	"Choi Jonghwan", 
	"http://saxboy.wo.to", 
	"saxboy@freechal.com", 
	__DATE__, 
	"yt_plugin_quicklaunch.png", 
	0, 
	210, 
	20, 
	yt_plugin_quicklaunch_initialize, 
	yt_plugin_quicklaunch_finalize, 
	yt_plugin_quicklaunch_update, 
	yt_plugin_quicklaunch_get_window, 
	yt_plugin_quicklaunch_get_config_window, 

	/* internal */
	NULL
}; 


YT_PLUGIN yt_plugin_get_info(void)
{
	return &g_yt_plugin_quicklaunch; 
}


/* -------------------------------------------------------------
 * internal plugin routines. 
 */

static GtkWidget *ytp_quicklaunch_window; 

static void on_toolbar_button_clicked( GtkWidget *widget, gpointer execpath)
{
	pid_t child_pid; 

	/* execute the program 
	 */
	execute_command_background( (const char *)execpath, &child_pid); 
	printf("ytp_quicklaunch_window: executed [%s]\n", (char *)execpath); 
}

static void _quicklaunch_initialize_append_button(
	const char *iconpath, const char *execpath)
{
	GtkWidget *icon_image_widget=NULL; 

	if (execpath==NULL) return; 

	//printf("loading icon[%s] with exec[%s]\n", iconpath, execpath); 

	/* replace iconpath with unknown.png 
	 * if iconpath is invalid or the image file cannot be loaded
	 */
	icon_image_widget = get_picture_pixmap_with_size((const char *)iconpath, 16, 16); 
	if (icon_image_widget==NULL) 
	{
		icon_image_widget = get_picture_pixmap_with_size(UNKNOWN_ICON_PATH, 16, 16); 
	}

	gtk_toolbar_append_item( 
		GTK_TOOLBAR(ytp_quicklaunch_window), 
		NULL, NULL, NULL, 
		icon_image_widget, 
		GTK_SIGNAL_FUNC( on_toolbar_button_clicked), 
		(gpointer) strdup(execpath) ); 


}

static void _quicklaunch_initialize_load_buttons(void)
{
	FILE *fp=NULL; 
	char buf[1024]={0,}; 

	if ( (fp = fopen(YT_PLUGIN_QUICKLAUNCH_CONFIG, "r")) ==NULL )
		return; 

	while ( fgets(buf, 1024, fp) !=NULL) 
	{
		GnomeDesktopEntry *dentry=NULL; 

		_confirm_no_blanks(buf, 1024); 
		if ( buf[0]=='\0' ) continue; 

//		printf("loading [%s]\n", buf); 	
		if ( (dentry = gnome_desktop_entry_load(buf))!=NULL) 
		{
//			printf("loaded dentry [%s]\n", dentry->location); 
			_quicklaunch_initialize_append_button( dentry->icon, dentry->exec[0]); 
		}
		gnome_desktop_entry_free(dentry); 
	}
	fclose(fp); 
}

static void yt_plugin_quicklaunch_initialize(void)
{
	/* we should have the configuration file
	 */
	if ( !file_Exist(YT_PLUGIN_QUICKLAUNCH_CONFIG) )
	{
		ytp_quicklaunch_window=NULL; 
		return; 
	}

	/* create plugin widgets
	 */
	ytp_quicklaunch_window = 
		gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH); 

	gtk_toolbar_set_button_relief( 
		GTK_TOOLBAR(ytp_quicklaunch_window), GTK_RELIEF_NONE); 

	/* append toolbar buttons 
	 */
	_quicklaunch_initialize_load_buttons(); 

	gtk_widget_show_all(ytp_quicklaunch_window); 

	yt_plugin_quicklaunch_update(); 
}

static void yt_plugin_quicklaunch_finalize(void)
{
	//gtk_widget_hide_all(ytp_quicklaunch_window); 
}

static void yt_plugin_quicklaunch_update(void)
{
	/* do nothing */
	return; 
}

static GtkWidget *yt_plugin_quicklaunch_get_window(void)
{
	return ytp_quicklaunch_window; 
}


/* -----------------------------------------------------------------
 * yopytoday plugin quicklaunch configuration 
 *
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>

static void _config_append_dentry_to_clist(
			GtkCList *clist, const char *desktopfile)
{
	GnomeDesktopEntry *dentry=NULL; 
	char *dummyentry[]={"xx","xx"}; 
	int cur_row_idx; 

	if (clist==NULL) return; 
	if (desktopfile==NULL) return; 

	if ( (dentry = gnome_desktop_entry_load(desktopfile))!=NULL) 
	{
		gtk_clist_append( clist, dummyentry); 
		cur_row_idx = (clist->rows)-1; 

		/* set icon pixmap 
		 */
		{
			GdkPixmap *pixmap=NULL, *bitmap=NULL; 
			GdkImlibImage *im=NULL; 

			if ( (im=gdk_imlib_load_image( dentry->icon))!=NULL)
			{
				gdk_imlib_render(im, 16, 16); 

				if ( (pixmap=gdk_imlib_move_image(im))!=NULL)
					gdk_pixmap_ref(pixmap); 
				if ( (bitmap=gdk_imlib_move_mask(im))!=NULL)
					gdk_pixmap_ref(bitmap); 

				gtk_clist_set_pixmap(clist, cur_row_idx, 0, pixmap, bitmap); 

				gdk_imlib_destroy_image(im); 
			}

		}

		/* set desktop entry name 
		 */
		gtk_clist_set_text(clist, cur_row_idx, 1, dentry->name); 

		/* set desktopfile as row data
		 */
		gtk_clist_set_row_data( clist, cur_row_idx, strdup(desktopfile)); 

//printf("append icon=%s, exec=%s, name=%s\n", dentry->icon, dentry->exec[0], dentry->name); 
	}
	gnome_desktop_entry_free(dentry); 
}

static void _config_scan_directory(GtkCList *clist, const char *dir)
{
	DIR *dp=NULL; 
	struct dirent *dirp=NULL; 

	if (dir==NULL) return; 
	if (clist==NULL) return; 

	if ( (dp=opendir(dir))==NULL) return; 

	while ( (dirp=readdir(dp))!=NULL)
	{
		char filename[1024]={0,}; 
		struct stat stbuf; 

		if ( strcmp(dirp->d_name, ".")==0 )
			continue; 
		if ( strcmp(dirp->d_name, "..")==0 )
			continue; 

		sprintf(filename, "%s/%s", dir, dirp->d_name); 

		if ( stat(filename, &stbuf) < 0 )
			continue; 

		if ( S_ISDIR(stbuf.st_mode) )
		{
			_config_scan_directory(clist, filename); 
		}
		else 
		{
			if ( strstr(filename, ".desktop")!=NULL )
				_config_append_dentry_to_clist(clist, filename); 
		}
	}

}


static GtkWidget *confwidget; 

static	GtkWidget *conf_vbox; 
static		GtkWidget *scroll_all; 
static 			GtkWidget *clist_all; 
static 		GtkWidget *button_hbox; 
static 			GtkWidget *button_enable; 
static 			GtkWidget *button_disable; 
static 			GtkWidget *button_save; 
static 		GtkWidget *scroll_selected; 
static 			GtkWidget *clist_selected; 


static int g_cur_idx_clist_all; 
static int g_cur_idx_clist_selected; 


static void _config_clist_selected_select_cb(
	GtkCList *clist, int row, int column, 
	GdkEventButton *event, gpointer user_data)
{
	g_cur_idx_clist_selected = row; 
}

static void _config_clist_all_select_cb(
	GtkCList *clist, int row, int column, 
	GdkEventButton *event, gpointer user_data)
{
	g_cur_idx_clist_all = row; 
}
	
static void _config_save_clicked_cb( GtkWidget *widget, gpointer data)
{
	FILE *fp=NULL; 
	int i=0; 
	
	if ( (fp = fopen(YT_PLUGIN_QUICKLAUNCH_CONFIG, "w+")) ==NULL )
		return; 

	for (i=0; i< GTK_CLIST( clist_selected)->rows; i++)
	{
		char *dtfilename=NULL; 

		dtfilename = 
			(char *)gtk_clist_get_row_data( GTK_CLIST(clist_selected), i); 

		if (dtfilename!=NULL)
			fprintf(fp, "%s\n", dtfilename); 
	}

	fclose(fp); 


	show_message(" Ǿϴ.\nyopytoday ٽ ϼ"); 
	
}
static void _config_enable_clicked_cb( GtkWidget *widget, gpointer data)
{
	char *desktopfilename=NULL; 

	/* get desktopfilename from clist_all 
	 */
	desktopfilename = (char *)gtk_clist_get_row_data( 
						GTK_CLIST(clist_all), g_cur_idx_clist_all); 

	/* append this entry to clist_selected
	 */
	_config_append_dentry_to_clist( 
		GTK_CLIST(clist_selected), desktopfilename); 
	
}
static void _config_disable_clicked_cb( GtkWidget *widget, gpointer data)
{
	gtk_clist_remove( GTK_CLIST(clist_selected), g_cur_idx_clist_selected); 
}

static GtkWidget *yt_plugin_quicklaunch_get_config_window(void)
{
	conf_vbox = gtk_vbox_new(FALSE, 0); 

	scroll_all = gtk_scrolled_window_new(NULL, NULL); 
	scroll_selected = gtk_scrolled_window_new(NULL, NULL); 

	gtk_scrolled_window_set_policy( 
		GTK_SCROLLED_WINDOW(scroll_all), 
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC); 
	gtk_scrolled_window_set_policy( 
		GTK_SCROLLED_WINDOW(scroll_selected), 
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC); 

	clist_all = gtk_clist_new(2); 
	gtk_clist_set_column_auto_resize( GTK_CLIST(clist_all),0, TRUE ); 
	gtk_clist_set_column_auto_resize( GTK_CLIST(clist_all),1, TRUE ); 

	clist_selected = gtk_clist_new(2); 
	gtk_clist_set_column_auto_resize( GTK_CLIST(clist_selected),0, TRUE ); 
	gtk_clist_set_column_auto_resize( GTK_CLIST(clist_selected),1, TRUE ); 

	gtk_scrolled_window_add_with_viewport(
		GTK_SCROLLED_WINDOW(scroll_all), clist_all); 
	gtk_scrolled_window_add_with_viewport(
		GTK_SCROLLED_WINDOW(scroll_selected), clist_selected); 

	gtk_signal_connect(
		GTK_OBJECT(clist_all), "select-row", 
		(GtkSignalFunc)_config_clist_all_select_cb, 
		NULL); 
	gtk_signal_connect(
		GTK_OBJECT(clist_selected), "select-row", 
		(GtkSignalFunc)_config_clist_selected_select_cb, 
		NULL); 

	/* buttons 
	 */
	button_hbox = gtk_hbox_new(TRUE, 1); 
	button_enable = gtk_button_new_with_imagefile( YT_RESOURCE_DIR "ytp_quicklaunch_enable.png"); 
	button_disable = gtk_button_new_with_imagefile( YT_RESOURCE_DIR "ytp_quicklaunch_disable.png"); 
	button_save= gtk_button_new_with_imagefile( YT_RESOURCE_DIR "ytp_quicklaunch_save.png"); 
	gtk_box_pack_start( GTK_BOX(button_hbox), button_enable, TRUE, TRUE, 0 ); 
	gtk_box_pack_start( GTK_BOX(button_hbox), button_disable, TRUE, TRUE, 0 ); 
	gtk_box_pack_start( GTK_BOX(button_hbox), button_save, TRUE, TRUE, 0 ); 

	gtk_signal_connect( 
		GTK_OBJECT(button_enable), "clicked", 
		(GtkSignalFunc) _config_enable_clicked_cb, NULL); 
	gtk_signal_connect( 
		GTK_OBJECT(button_disable), "clicked", 
		(GtkSignalFunc) _config_disable_clicked_cb, NULL); 
	gtk_signal_connect( 
		GTK_OBJECT(button_save), "clicked", 
		(GtkSignalFunc) _config_save_clicked_cb, NULL); 

	/* finally pack all the widgets to the vbox 
	 */
	gtk_box_pack_start( GTK_BOX(conf_vbox), scroll_all, TRUE, TRUE, 0 ); 
	gtk_box_pack_start( GTK_BOX(conf_vbox), button_hbox, FALSE, FALSE, 0 ); 
	gtk_box_pack_start( GTK_BOX(conf_vbox), scroll_selected, TRUE, TRUE, 0 ); 

	/* scan desktop files 
	 */
	_config_scan_directory( GTK_CLIST(clist_all), "/usr/share/gnome/apps"); 

	/* append current setups to clist_selected
	 */
	{
		FILE *fp=NULL; 

		if ( (fp = fopen(YT_PLUGIN_QUICKLAUNCH_CONFIG, "r")) !=NULL )
		{
			char buf[1024]={0,}; 

			while ( fgets(buf, 1024, fp) !=NULL) 
			{
				_confirm_no_blanks(buf, 1024); 
				if ( !isspace(buf[0]) )
				{
					_config_append_dentry_to_clist( 
						GTK_CLIST(clist_selected), buf); 

				}
			}
		}
		fclose(fp); 
	}



	confwidget = conf_vbox; 

	return confwidget; 
}

