/*
 * sample plugin for yopytoday
 *
 * Choi Jonghwan <saxboy@freechal.com>
 * 2003/04/11
 */

#include <stdio.h>
#include <mysql.h>
#include "ytplugin.h"
#include "yopytoday_resource.h"
#include "util.h"

static void yt_plugin_todo_initialize(void); 
static void yt_plugin_todo_finalize(void); 
static void yt_plugin_todo_update(void); 
static GtkWidget *yt_plugin_todo_get_window(void); 
static GtkWidget *yt_plugin_todo_get_config_window(void); 


_YT_PLUGIN g_yt_plugin_todo = 
{
	"yopytoday_todo_plugin", 
	"0.01", 
	"Choi Jonghwan", 
	"http://saxboy.wo.to", 
	"saxboy@freechal.com", 
	__DATE__, 
	"yt_plugin_todo.png", 
	10000, 
	210, 
	80, 
	yt_plugin_todo_initialize, 
	yt_plugin_todo_finalize, 
	yt_plugin_todo_update, 
	yt_plugin_todo_get_window, 
	yt_plugin_todo_get_config_window, 
	
	/* internal */
	NULL
}; 


YT_PLUGIN yt_plugin_get_info(void)
{
	return &g_yt_plugin_todo; 
}


/* -------------------------------------------------------------
 * internal plugin routines. 
 */

static GtkWidget *ytp_todo_window; 
static GtkWidget *ytp_todo_clist; 
static MYSQL ytp_todo_mysql; 

static char ytp_todo_query[1024]; 

#define YTP_TODO_CONFIG 	YT_PLUGIN_DIR "/yt_plugin_todo.query"
#define YTP_TODO_QUERY_DEFAULT	\
	"select dueDate, subject from Task where ( startDate >= CURRENT_DATE and startDate<= CURRENT_DATE()+1 ) or ( startDate <= CURRENT_DATE and completed='0' )"


static void yt_plugin_todo_initialize(void)
{

	/* create plugin widgets
	 */

	ytp_todo_window = gtk_scrolled_window_new(NULL, NULL); 
	gtk_scrolled_window_set_policy(
		GTK_SCROLLED_WINDOW(ytp_todo_window), 
		GTK_POLICY_NEVER, 
		GTK_POLICY_AUTOMATIC); 

	ytp_todo_clist = gtk_clist_new(2); 

	/* read todo items from mysql & append it to the clist
	 */
#if 0
	gtk_clist_set_column_title( GTK_CLIST(ytp_todo_clist), 0, "priority" ); 
	gtk_clist_set_column_title( GTK_CLIST(ytp_todo_clist), 1, "todo" ); 
	gtk_clist_column_titles_show( GTK_CLIST(ytp_todo_clist)); 
#endif
	gtk_clist_set_column_auto_resize( GTK_CLIST(ytp_todo_clist), 0, TRUE); 
	gtk_clist_set_column_auto_resize( GTK_CLIST(ytp_todo_clist), 1, TRUE); 


	gtk_container_add( GTK_CONTAINER(ytp_todo_window), ytp_todo_clist); 
	gtk_widget_show_all(ytp_todo_window); 

	/* initialize query 
	 */
	{
		FILE *fp=NULL; 
		if ( (fp=fopen(YTP_TODO_CONFIG, "r"))!=NULL)
		{
			fgets( ytp_todo_query, 1024, fp); 
			fclose(fp); 
		}
	}
	if ( ytp_todo_query[0]=='\0' )
		strncpy( ytp_todo_query, YTP_TODO_QUERY_DEFAULT, 1024); 

	printf("todo: initailized query [%s]\n", ytp_todo_query); 

	/* connect to the database first
	 */
	if ( mysql_init( &ytp_todo_mysql) == NULL )
		return;
	if ( !mysql_real_connect( &ytp_todo_mysql, "localhost", "root", "", "YopimDB", 3306, NULL, 0 ))      
		return; 


	yt_plugin_todo_update(); 
}

static void yt_plugin_todo_finalize(void)
{
	//gtk_widget_hide_all(ytp_todo_window); 

	/* close database 
	 */
	mysql_close( &ytp_todo_mysql); 
}

static void  _todo_append_entry(const char *date, const char *text)
{
	char *textarr[2] = {NULL, }; 

//printf("append todo [%s:%s]\n", date, text); 

	if ( date!=NULL) textarr[0] = strdup(date); 
	if ( text!=NULL) textarr[1] = strdup(text); 
	gtk_clist_append( GTK_CLIST(ytp_todo_clist), textarr); 

}
static void yt_plugin_todo_update(void)
{
//	printf("update todo\n"); 

#if 0
#define YTP_TODO_QUERY	"select dueDate, subject from Task "
#endif
	/* send query to the database 
	 */
	if ( mysql_query( &ytp_todo_mysql, ytp_todo_query ) )
		return; 

	/* refresh the widget & append results to the list
	 */
	gtk_clist_clear( GTK_CLIST( ytp_todo_clist) ); 
	{
		MYSQL_RES *res=NULL; 
		MYSQL_ROW row; 
		int field_cnt=0; 
		int row_cnt=0; 

		res = mysql_store_result( &ytp_todo_mysql); 
		field_cnt = mysql_num_fields(res); /* should be 2 */
		row_cnt = mysql_num_rows(res); 

		while ( (row=mysql_fetch_row(res)) )
		{
			_todo_append_entry( row[0], row[1] ); 
		}
	}

}

static GtkWidget *yt_plugin_todo_get_window(void)
{
	return ytp_todo_window; 
}

static GtkWidget *ytp_todo_config_window; 

static 	GtkWidget *config_vbox; 
static 		GtkWidget *button_hbox; 
static 			GtkWidget *button_default; 
static 			GtkWidget *button_save; 
static 		GtkWidget *query_text; 


static void _config_default_clicked_cb( GtkWidget *widget, gpointer data)
{
	int pos=0; 

	gtk_editable_delete_text(GTK_EDITABLE(query_text), 0, -1); 
	gtk_editable_insert_text( 
		GTK_EDITABLE(query_text), 
			YTP_TODO_QUERY_DEFAULT, strlen(YTP_TODO_QUERY_DEFAULT), &pos); 
}

static void _config_save_clicked_cb( GtkWidget *widget, gpointer data)
{
    FILE *fp=NULL;
	gchar *query_string = NULL; 

	query_string = gtk_editable_get_chars( GTK_EDITABLE(query_text), 0, -1); 
	if ( query_string==NULL)
	{
    	show_message("˼  ߻Ͽϴ."); 
		return; 
	}

	if ( (fp=fopen(YTP_TODO_CONFIG, "w+"))==NULL)
	{
    	show_message("   ϴ"); 
        return; 
	}

	fprintf(fp, "%s\n", query_string); 

	fclose(fp);


    show_message(" Ǿϴ.\nyopytoday ٽ ϼ");
}

static GtkWidget *yt_plugin_todo_get_config_window(void)
{
	config_vbox = gtk_vbox_new(FALSE, 0); 

	button_hbox = gtk_hbox_new(TRUE, 1 ); 
	button_default = gtk_button_new_with_imagefile( YT_RESOURCE_DIR "ytp_todo_default.png"); 
	button_save = gtk_button_new_with_imagefile( YT_RESOURCE_DIR "ytp_todo_save.png"); 

	gtk_signal_connect(
		GTK_OBJECT(button_default), "clicked", 
		(GtkSignalFunc) _config_default_clicked_cb, NULL); 
	gtk_signal_connect(
		GTK_OBJECT(button_save), "clicked", 
		(GtkSignalFunc) _config_save_clicked_cb, NULL); 

	gtk_box_pack_start(GTK_BOX(button_hbox), button_default, TRUE, TRUE, 0 ); 
	gtk_box_pack_start(GTK_BOX(button_hbox), button_save, TRUE, TRUE, 0 ); 

	query_text = gtk_text_new(NULL, NULL); 
	gtk_text_set_editable( GTK_TEXT(query_text), TRUE); 
	gtk_text_set_word_wrap( GTK_TEXT(query_text), TRUE); 

	gtk_box_pack_start(GTK_BOX(config_vbox), button_hbox, FALSE, FALSE, 0 ); 
	gtk_box_pack_start(GTK_BOX(config_vbox), query_text, TRUE, TRUE, 0 ); 

	/* set default configuraion
	 */
	{
		FILE *fp=NULL; 
		char query_string[1024]={0,}; 
		int pos=0; 

		if ( (fp=fopen(YTP_TODO_CONFIG, "r"))!=NULL)
		{
			fgets( query_string, 1024, fp); 
			fclose(fp); 
		}
		if ( query_string[0]=='\0' )
			strncpy( query_string, YTP_TODO_QUERY_DEFAULT, 1024); 

		gtk_editable_insert_text( 
			GTK_EDITABLE(query_text), 
			query_string, strlen(query_string), &pos); 
	}


	ytp_todo_config_window = config_vbox; 
	return ytp_todo_config_window; 
}
